@@ -4,25 +4,22 @@ implementation written in Node.js).
 """
 
 from ape import plugins
-from ape.api.networks import LOCAL_NETWORK_NAME
-from ape_ethereum.ecosystem import NETWORKS
-
-from .provider import (
-    FoundryForkProvider,
-    FoundryNetworkConfig,
-    FoundryProvider,
-    FoundryProviderError,
-    FoundrySubprocessError,
-)
 
 
 @plugins.register(plugins.Config)
 def config_class():
+    from .provider import FoundryNetworkConfig
+
     return FoundryNetworkConfig
 
 
 @plugins.register(plugins.ProviderPlugin)
 def providers():
+    from ape.api.networks import LOCAL_NETWORK_NAME
+    from ape_ethereum.ecosystem import NETWORKS
+
+    from .provider import FoundryForkProvider, FoundryProvider
+
     yield "ethereum", LOCAL_NETWORK_NAME, FoundryProvider
 
     for network in NETWORKS:
@@ -61,7 +58,38 @@ def providers():
     yield "blast", "sepolia-fork", FoundryForkProvider
 
 
+def __getattr__(name: str):
+    if name == "FoundryForkProvider":
+        from ape_foundry.provider import FoundryForkProvider
+
+        return FoundryForkProvider
+
+    elif name == "FoundryNetworkConfig":
+        from ape_foundry.provider import FoundryNetworkConfig
+
+        return FoundryNetworkConfig
+
+    elif name == "FoundryProvider":
+        from ape_foundry.provider import FoundryProvider
+
+        return FoundryProvider
+
+    elif name == "FoundryProviderError":
+        from ape_foundry.provider import FoundryProviderError
+
+        return FoundryProviderError
+
+    elif name == "FoundrySubprocessError":
+        from ape_foundry.provider import FoundrySubprocessError
+
+        return FoundrySubprocessError
+
+    else:
+        raise AttributeError(name)
+
+
 __all__ = [
+    "FoundryForkProvider",
     "FoundryNetworkConfig",
     "FoundryProvider",
     "FoundryProviderError",
