@@ -3,7 +3,7 @@ import random
 import shutil
 from bisect import bisect_right
 from subprocess import PIPE, call
-from typing import Literal, Optional, Union, cast
+from typing import TYPE_CHECKING, Literal, Optional, Union, cast
 
 from ape.api import (
     BlockAPI,
@@ -23,7 +23,6 @@ from ape.exceptions import (
     VirtualMachineError,
 )
 from ape.logging import logger
-from ape.types import AddressType, BlockID, ContractCode, SnapshotID
 from ape.utils import cached_property
 from ape_ethereum.provider import Web3Provider
 from ape_ethereum.trace import TraceApproach, TransactionTrace
@@ -38,7 +37,11 @@ from web3.exceptions import ContractCustomError
 from web3.exceptions import ContractLogicError as Web3ContractLogicError
 from web3.exceptions import ExtraDataLengthError
 from web3.gas_strategies.rpc import rpc_gas_price_strategy
-from web3.middleware import geth_poa_middleware
+
+try:
+    from web3.middleware import ExtraDataToPOAMiddleware  # type: ignore
+except ImportError:
+    from web3.middleware import geth_poa_middleware as ExtraDataToPOAMiddleware  # type: ignore
 from web3.middleware.validation import MAX_EXTRADATA_LENGTH
 from yarl import URL
 
@@ -54,6 +57,10 @@ try:
 except ImportError:
     Optimism = None  # type: ignore
 
+if TYPE_CHECKING:
+    from ape.types import AddressType, BlockID, ContractCode, SnapshotID
+
+
 EPHEMERAL_PORTS_START = 49152
 EPHEMERAL_PORTS_END = 60999
 DEFAULT_PORT = 8545
@@ -136,7 +143,7 @@ class FoundryProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
     _did_warn_wrong_node = False
 
     @property
-    def unlocked_accounts(self) -> list[AddressType]:
+    def unlocked_accounts(self) -> list["AddressType"]:
         return list(self.account_manager.test_accounts._impersonated_accounts)
 
     @property
@@ -397,7 +404,7 @@ class FoundryProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
 
         # Handle if using PoA
         if any(map(check_poa, (0, "latest"))):
-            self._web3.middleware_onion.inject(geth_poa_middleware, layer=0)
+            self._web3.middleware_onion.inject(ExtraDataToPOAMiddleware, layer=0)
 
     def _start(self):
         if self.is_connected:
@@ -485,7 +492,7 @@ class FoundryProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
 
         return cmd
 
-    def set_balance(self, account: AddressType, amount: Union[int, float, str, bytes]):
+    def set_balance(self, account: "AddressType", amount: Union[int, float, str, bytes]):
         is_str = isinstance(amount, str)
         is_key_word = is_str and " " in amount  # type: ignore
         _is_hex = is_str and not is_key_word and amount.startswith("0x")  # type: ignore
@@ -516,19 +523,19 @@ class FoundryProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
     def snapshot(self) -> str:
         return self.make_request("evm_snapshot", [])
 
-    def restore(self, snapshot_id: SnapshotID) -> bool:
+    def restore(self, snapshot_id: "SnapshotID") -> bool:
         snapshot_id = to_hex(snapshot_id) if isinstance(snapshot_id, int) else snapshot_id
         result = self.make_request("evm_revert", [snapshot_id])
         return result is True
 
-    def unlock_account(self, address: AddressType) -> bool:
+    def unlock_account(self, address: "AddressType") -> bool:
         self.make_request("anvil_impersonateAccount", [address])
         return True
 
-    def relock_account(self, address: AddressType):
+    def relock_account(self, address: "AddressType"):
         self.make_request("anvil_stopImpersonatingAccount", [address])
 
-    def get_balance(self, address: AddressType, block_id: Optional[BlockID] = None) -> int:
+    def get_balance(self, address: "AddressType", block_id: Optional["BlockID"] = None) -> int:
         if result := self.make_request("eth_getBalance", [address, block_id]):
             return int(result, 16) if isinstance(result, str) else result
 
@@ -546,88 +553,82 @@ class FoundryProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
         return _get_transaction_trace(transaction_hash, **kwargs)
 
     def get_virtual_machine_error(self, exception: Exception, **kwargs) -> VirtualMachineError:
-        if not len(exception.args):
+        if not exception.args:
             return VirtualMachineError(base_err=exception, **kwargs)
 
         err_data = exception.args[0]
 
+        # Determine message based on the type of error data
         if isinstance(err_data, dict):
             message = str(err_data.get("message", f"{err_data}"))
-        elif isinstance(err_data, str):
-            message = err_data
-        elif msg := getattr(exception, "message", ""):
-            message = msg
         else:
-            message = ""
+            message = err_data if isinstance(err_data, str) else getattr(exception, "message", "")
 
         if not message:
             return VirtualMachineError(base_err=exception, **kwargs)
 
-        def _handle_execution_reverted(
-            exception: Exception, revert_message: Optional[str] = None, **kwargs
-        ):
-            if revert_message in ("", "0x", None):
-                revert_message = TransactionError.DEFAULT_MESSAGE
-
-            sub_err = ContractLogicError(
-                base_err=exception, revert_message=revert_message, **kwargs
-            )
-            enriched = self.compiler_manager.enrich_error(sub_err)
-
-            # Show call trace if available
-            if enriched.txn:
-                # Unlikely scenario where a transaction is on the error even though a receipt
-                # exists.
-                if isinstance(enriched.txn, TransactionAPI) and enriched.txn.receipt:
-                    try:
-                        enriched.txn.receipt.show_trace()
-                    except Exception:
-                        # TODO: Fix this in Ape
-                        pass
-
-                elif isinstance(enriched.txn, ReceiptAPI):
-                    enriched.txn.show_trace()
-
-            return enriched
-
-        # Handle `ContactLogicError` similarly to other providers in `ape`.
-        # by stripping off the unnecessary prefix that foundry has on reverts.
+        # Handle specific cases based on message content
         foundry_prefix = (
             "Error: VM Exception while processing transaction: reverted with reason string "
         )
+
+        # Handle Foundry error prefix
         if message.startswith(foundry_prefix):
             message = message.replace(foundry_prefix, "").strip("'")
-            return _handle_execution_reverted(exception, message, **kwargs)
+            return self._handle_execution_reverted(exception, message, **kwargs)
 
-        elif "Transaction reverted without a reason string" in message:
-            return _handle_execution_reverted(exception, **kwargs)
+        # Handle various cases of transaction reverts
+        if "Transaction reverted without a reason string" in message:
+            return self._handle_execution_reverted(exception, **kwargs)
 
-        elif message.lower() == "execution reverted":
+        if message.lower() == "execution reverted":
             message = TransactionError.DEFAULT_MESSAGE
-            if isinstance(exception, Web3ContractLogicError) and (
-                msg := self._extract_custom_error(**kwargs)
-            ):
-                exception.message = msg
+            if isinstance(exception, Web3ContractLogicError):
+                if custom_msg := self._extract_custom_error(**kwargs):
+                    exception.message = custom_msg
+            return self._handle_execution_reverted(exception, revert_message=message, **kwargs)
 
-            return _handle_execution_reverted(exception, revert_message=message, **kwargs)
-
-        elif message == "Transaction ran out of gas" or "OutOfGas" in message:
+        if "Transaction ran out of gas" in message or "OutOfGas" in message:
             return OutOfGasError(base_err=exception, **kwargs)
 
-        elif message.startswith("execution reverted: "):
+        if message.startswith("execution reverted: "):
             message = (
                 message.replace("execution reverted: ", "").strip()
                 or TransactionError.DEFAULT_MESSAGE
             )
-            return _handle_execution_reverted(exception, revert_message=message, **kwargs)
+            return self._handle_execution_reverted(exception, revert_message=message, **kwargs)
 
-        elif isinstance(exception, ContractCustomError):
-            # Is raw hex (custom exception)
+        # Handle custom errors
+        if isinstance(exception, ContractCustomError):
             message = TransactionError.DEFAULT_MESSAGE if message in ("", None, "0x") else message
-            return _handle_execution_reverted(exception, revert_message=message, **kwargs)
+            return self._handle_execution_reverted(exception, revert_message=message, **kwargs)
 
         return VirtualMachineError(message, **kwargs)
 
+    # The type ignore is because are using **kwargs rather than repeating.
+    def _handle_execution_reverted(  # type: ignore[override]
+        self, exception: Exception, revert_message: Optional[str] = None, **kwargs
+    ):
+        # Assign default message if revert_message is invalid
+        if revert_message == "0x":
+            revert_message = TransactionError.DEFAULT_MESSAGE
+        else:
+            revert_message = revert_message or TransactionError.DEFAULT_MESSAGE
+
+        # Create and enrich the error
+        sub_err = ContractLogicError(base_err=exception, revert_message=revert_message, **kwargs)
+        enriched = self.compiler_manager.enrich_error(sub_err)
+
+        # Show call trace if available
+        txn = enriched.txn
+        if txn and hasattr(txn, "show_trace"):
+            if isinstance(txn, TransactionAPI) and txn.receipt:
+                txn.receipt.show_trace()
+            elif isinstance(txn, ReceiptAPI):
+                txn.show_trace()
+
+        return enriched
+
     # Abstracted for easier testing conditions.
     def _extract_custom_error(self, **kwargs) -> str:
         # Check for custom error.
@@ -643,19 +644,15 @@ class FoundryProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
             except Exception:
                 pass
 
-        try:
-            if trace is not None and (revert_msg := trace.revert_message):
-                return revert_msg
-        except Exception:
-            # TODO: Fix this in core
-            return ""
+        if trace is not None and (revert_msg := trace.revert_message):
+            return revert_msg
 
         return ""
 
     def set_block_gas_limit(self, gas_limit: int) -> bool:
         return self.make_request("evm_setBlockGasLimit", [hex(gas_limit)]) is True
 
-    def set_code(self, address: AddressType, code: ContractCode) -> bool:
+    def set_code(self, address: "AddressType", code: "ContractCode") -> bool:
         if isinstance(code, bytes):
             code = code.hex()
 
@@ -668,7 +665,7 @@ class FoundryProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
         self.make_request("anvil_setCode", [address, code])
         return True
 
-    def set_storage(self, address: AddressType, slot: int, value: HexBytes):
+    def set_storage(self, address: "AddressType", slot: int, value: HexBytes):
         self.make_request(
             "anvil_setStorageAt",
             [
@@ -722,7 +719,7 @@ class FoundryForkProvider(FoundryProvider):
 
         return self.settings.evm_version
 
-    def get_block(self, block_id: BlockID) -> BlockAPI:
+    def get_block(self, block_id: "BlockID") -> BlockAPI:
         if isinstance(block_id, str) and block_id.isnumeric():
             block_id = int(block_id)
 
@@ -793,7 +790,9 @@ class FoundryForkProvider(FoundryProvider):
                     logger.error(
                         f"Upstream provider '{upstream_provider.name}' missing Geth PoA middleware."
                     )
-                    upstream_provider.web3.middleware_onion.inject(geth_poa_middleware, layer=0)
+                    upstream_provider.web3.middleware_onion.inject(
+                        ExtraDataToPOAMiddleware, layer=0
+                    )
                     upstream_genesis_block_hash = upstream_provider.get_block(0).hash
                 else:
                     raise FoundryProviderError(f"Unable to get genesis block: {err}.") from err
